type ctx = (* inherited debug info *)
    {
      ctx_step :int;
      ctx_name :string;
      ctx_depth:int;
      ctx_data: Data.subst list;
      ctx_terminate: unit -> unit;
      ctx_inputs  : string list;
      ctx_outputs : string list;
    }


type src_info = { 
  str : string ;
  file : string ; 
  line : int * int ;
  char : int * int ;
  stack : src_info option;
}


(* Source info related to the elected constraint *)
type cstr_lazy_src_info = (unit -> src_info list)
type port = 
  | Call 
  | Exit of string * Expr.t (* The elected contraint that produced the outputs *) * cstr_lazy_src_info
  | Fail of string * Expr.t (* The elected contraint that failed *) * cstr_lazy_src_info * 
      (unit -> Failure.info)


type node_info = {
  port : port;
  name : string;
  lang : string; 
  inputs  : string list; (* static info *)
  outputs : string list; (* static info *)
}

type kind = 
  | Node of node_info
  | Ltop

(* when an elected constraint is unsatisfiable *)

type t = { 
  nb:int;
  step  : int;
  depth : int;
  kind  : kind;
  data  : Data.subst list;  
  other : string;
  next  : unit -> t;
  terminate: unit -> unit;
} 
    (* ou bien un MapString ?
       ou bien une structure avec tout un tas d'autre info ?
       devra etre abrait en tout cas
    *)


exception End of int
let (next : t -> t) = fun e -> e.next ()
let (terminate : t -> unit) = fun e -> e.terminate ()
let (data : t -> Data.subst list) = fun e -> e.data


let seed = ref 42
let set_seed s = seed := s

let event_nb = ref 0
let set_nb i = event_nb := i
let get_nb () =  !event_nb
let incr_nb() =
  Random.init (!seed + !event_nb); (* make sure everything is reproductible *)
  incr event_nb



    
